@@ -64,6 +64,10 @@ namespace xt
     template <class C>
     using get_stepper_iterator = typename detail::get_stepper_iterator_impl<C>::type;
 
+    /********************************
+     * xindex_type_t implementation *
+     ********************************/
+
     namespace detail
     {
         template <class ST>
@@ -77,6 +81,12 @@ namespace xt
         {
             using type = std::array<V, L>;
         };
+
+        template <std::size_t... I>
+        struct index_type_impl<fixed_shape<I...>>
+        {
+            using type = std::array<std::size_t, sizeof...(I)>;
+        };
     }
 
     template <class C>
@@ -380,25 +390,25 @@ namespace xt
     namespace detail
     {
         template <class C>
-        constexpr auto trivial_begin(C& c) noexcept
+        XTENSOR_CONSTEXPR_RETURN auto trivial_begin(C& c) noexcept
         {
             return c.storage_begin();
         }
 
         template <class C>
-        constexpr auto trivial_end(C& c) noexcept
+        XTENSOR_CONSTEXPR_RETURN auto trivial_end(C& c) noexcept
         {
             return c.storage_end();
         }
 
         template <class C>
-        constexpr auto trivial_begin(const C& c) noexcept
+        XTENSOR_CONSTEXPR_RETURN auto trivial_begin(const C& c) noexcept
         {
             return c.storage_begin();
         }
 
         template <class C>
-        constexpr auto trivial_end(const C& c) noexcept
+        XTENSOR_CONSTEXPR_RETURN auto trivial_end(const C& c) noexcept
         {
             return c.storage_end();
         }
